#include <Alert.h>
#include <Directory.h>
#include <Entry.h>
#include <Font.h>
#include <ListItem.h>
#include <MenuItem.h>
#include <Node.h>
#include <NodeInfo.h>
#include <Path.h>
#include <Point.h>
#include <PopUpMenu.h>
#include <String.h>
#include <Window.h>

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "PlayWindow.h"
#include "SongListView.h"

SongListView::SongListView(BRect frame)
	: BListView(frame, NULL, B_MULTIPLE_SELECTION_LIST, B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS)
{
	// Seed this for the random movement
	srand(time(NULL));
	
	// Build the pop-up menu for right-clicking on items
	PopUp = new BPopUpMenu("popmenu", false, false);
	PopUp->AddItem(new BMenuItem("Move Up", new BMessage(MOVE_UP)));
	PopUp->AddItem(new BMenuItem("Move Down", new BMessage(MOVE_DOWN)));
	PopUp->AddItem(new BMenuItem("Remove", new BMessage(REMOVE)));
	PopUp->AddItem(new BMenuItem("Play", new BMessage(PLAY)));

	// Small fonts are cool
	BFont font;
	PopUp->GetFont(&font);
	font.SetSize(10);
	PopUp->SetFont(&font);
}

void SongListView::AttachedToWindow()
{
	if (Parent())
		PopUp->SetTargetForItems(this);
}

void SongListView::SelectionChanged()
{
	PlayWindow *parent = (PlayWindow *)Window();
	if (CurrentSelection() >= 0)
		parent->EnablePlayButton(true);
	else
		parent->EnablePlayButton(false);
}

void SongListView::KeyDown(const char *bytes, int32 numBytes)
{
	switch (bytes[0])
	{
		case B_BACKSPACE:
		case B_DELETE:
			MessageReceived(new BMessage(REMOVE));			
			break;
		
		default:
			BListView::KeyDown(bytes, numBytes);
	}
}

void SongListView::MouseDown(BPoint where)
{
	BPoint pt;
	uint32 buttons;
	GetMouse(&pt, &buttons);
	
	if (buttons == B_SECONDARY_MOUSE_BUTTON) {
		ConvertToScreen(&where);
		PopUp->Go(where, true, false, true);
	} else { 
		BListView::MouseDown(where);
	}
}

void SongListView::MessageReceived(BMessage *msg)
{
	switch (msg->what)
	{
		case B_SIMPLE_DATA:
		{
			entry_ref ref;
			for (int32 i=0; msg->FindRef("refs", i, &ref) == B_OK; i++)
				AddSongs(ref);
		}
			break;
			
		case CLEAR:
		{
			PlayWindow *parent = (PlayWindow *)Window();
			while (CountItems() > 0)
				delete RemoveItem((int32)0);
			parent->EnableSaveButton(false);
		}
			break;
		
		case MOVE_UP:
		{
			int32 i, selected;
			for (i=0; (selected = CurrentSelection(i)) > 0; i++)
				MoveItem(selected, selected-1);
			ScrollToSelection();
		}	
			break;
		
		case MOVE_DOWN:
		{
			int32 i, max, selected;

			// Going in reverse is tricky; find the total selections
			for (max=0; CurrentSelection(max) >= 0; max++)
				continue;
			
			// Now start at the end and work backwards
			for (i=max; i>=0; i--) {
				selected = CurrentSelection(i);
				if (selected < CountItems()-(max-i)) {
					MoveItem(selected, selected+1);
				}
			}
			ScrollToSelection();
		}			
			break;

		case REMOVE:
		{
			// Delete every selected item
			int32 selected = 0;
			while (CurrentSelection(0) >= 0) {
				selected = CurrentSelection(0);
				delete RemoveItem(selected);
	
				if (selected >= CountItems()) 
					selected = CountItems() - 1;
			}
	
			// Select whatever was left
			Select(selected);
			ScrollToSelection();
			
			// Disable the save button if nothing is left
			PlayWindow *parent = (PlayWindow *)Window();
			if (CountItems() < 1)
				parent->EnableSaveButton(false);
		}
			break;

		case SORT_PATH:
			SortItems(sort_path);
			break;
			
		case SORT_TITLE:
			SortItems(sort_title);
			break;
			
		case RANDOMIZE:
			for (int32 i=0; i<CountItems(); i++)
				Randomize(i);
			break;

		default:
			BListView::MessageReceived(msg);
	}
}

void SongListView::AddSongs(entry_ref ref)
{
	BEntry entry;
	BPath path;
	BNode node;
	BNodeInfo nodeInfo;

	entry.SetTo(&ref, true);
	if (entry.IsDirectory()) {
		// Recursively search through directories
		BDirectory someDir(&entry);
		while (someDir.GetNextEntry(&entry) != B_ENTRY_NOT_FOUND) {
			entry_ref newref;
			entry.GetRef(&newref);
			AddSongs(newref);
		}
	} else {
		// Check that the file is an audio file
		char temp[B_MIME_TYPE_LENGTH];
		node.SetTo(&entry);
		nodeInfo.SetTo(&node);
		nodeInfo.GetType(temp);
		
		BString type = temp;
		if (type.FindFirst("audio") == 0) {
			path.SetTo(&entry);
			AddItem(new BStringItem(path.Path()));
		}
	}
	
	// Enable the Save button if there are any songs
	PlayWindow *parent = (PlayWindow *)Window();
	if (CountItems() > 0)
		parent->EnableSaveButton(true);
}

void SongListView::Randomize(int32 index)
{
	MoveItem(index, rand() % CountItems());
}

int sort_path(const void *a, const void *b)
{
	return strcmp((*(BStringItem **)a)->Text(), (*(BStringItem **)b)->Text());
}

int sort_title(const void *a, const void *b)
{
	BString one = (*(BStringItem **)a)->Text();
	BString two = (*(BStringItem **)b)->Text();

	// Trim off everything up to the filename
	one.Remove(0, one.FindLast("/"));
	two.Remove(0, two.FindLast("/"));

	// Compare!
	return strcmp(one.String(), two.String());
}
